#include "ADS131A0x.h"


/** Class constructor, only assigns parameters.
 *
 * @param spi			Pointer to SPI peripheral: max clock speed 25 MHz, 8 bit word size, mode 1 (CPHA = 1).
 * @param cs_port		Pointer to port of CS pin.
 * @param cs_pin		Mask of CS pin.
 * @param rst_port		Pointer to port of RST pin.
 * @param rst_pin		Mask of RST pin.
 * @param m1			Value fo M1 pin (determines number of bytes in a word), see the resúective enum.
 */
ADS131A0x::ADS131A0x(SPI_HandleTypeDef* spi, GPIO_TypeDef* cs_port, uint16_t cs_pin, GPIO_TypeDef* rst_port, uint16_t rst_pin, M1 m1):
		m_spi(spi), m_cs_port(cs_port), m_cs_pin(cs_pin), m_rst_port(rst_port), m_rst_pin(rst_pin), m_wordSizeInBytes((uint8_t)m1)
{

};


/** Initializes and unlocks the chip.
 *
 * @return				True if connection with the chip was established.
 */
bool ADS131A0x::init()
{
	reset();

	auto start = HAL_GetTick();
	uint16_t response = 0;
	while (!((response == 0xFF04) || (response == 0xFF02)))		//04 for 4 channel version
	{
		response = sendCommand(Command::NOP);
		HAL_Delay(1);

		if (HAL_GetTick() - start >= 100)
			return false;
	}

	sendCommand(Command::UNLOCK);
	response = sendCommand(Command::NOP);
	return (response == (uint16_t)Command::UNLOCK);
}


/** Resets the chip using RST pin.
 *
 */
void ADS131A0x::reset(void) const
{
	HAL_GPIO_WritePin(m_rst_port, m_rst_pin, GPIO_PIN_RESET);
	HAL_Delay(1);
	HAL_GPIO_WritePin(m_rst_port, m_rst_pin, GPIO_PIN_SET);
}


/** Configures clock source and dividers, see datasheet page 27.
 *
 * @param clkSrc		Clock source, see the enum.
 * @param clkIn			First clock divider, called CLK_DIV in datasheet.
 * @param iClk			Second clock divider, called ICLK_DIV in the datasheet.
 * @param osr			Oversampling ratio.
 *
 * @note				To calculate data rate: Fdata = Fclock / clkIn / iClk / osr.
 */
void ADS131A0x::setClockSource(ClkSrc clkSrc, ClkDiv clkIn, ClkDiv iClk, Osr osr) const
{
	uint8_t clk1 = (uint8_t)clkIn << 1;
	if (clkSrc == ClkSrc::SCLK)
		clk1 |= 0b10000000;

	uint8_t clk2 = ((uint8_t)iClk << 5) | (uint8_t)osr;

	writeRegister(Reg::CLK1, clk1);
	writeRegister(Reg::CLK2, clk2);
}


/** Configures reference voltage and fault threshold, which will raise flags in the STAT_P/N registers.
 *
 * @param reference			See the enum.
 * @param faultThreshold	Percentage of full range, see the enum.
 */
void ADS131A0x::setReferenceVoltage(Reference reference, FaultThreshold faultThreshold) const
{
	uint8_t cfg = 0b01100000 | ((uint8_t)faultThreshold);
	if (reference != Reference::EXTERNAL)
		cfg |= 0b1000;
	if (reference == Reference::INTERNAL_4000_mV)
		cfg |= 0b10000;

	writeRegister(Reg::A_SYS_CFG, cfg);
}


/** Configures channels of the ADC.
 *
 * @param ch1				Gain of channel 1.
 * @param ch2				Gain of channel 2.
 * @param ch3				Gain of channel 3.
 * @param ch4				Gain of channel 4.
 */
void ADS131A0x::enableChannels(Gain ch1, Gain ch2, Gain ch3, Gain ch4)
{
	/*m_channelGains[0] = ch1;
	m_channelGains[1] = ch2;
	m_channelGains[2] = ch3;
	m_channelGains[3] = ch4;*/

	uint8_t channelsMask = 0;
	m_activeChannels = 0;

	if (ch1 != Gain::DISABLED)
	{
		writeRegister(Reg::ADC_1, (uint8_t)ch1);
		m_activeChannels += 1;
		channelsMask |= (1 << 0);
	}

	if (ch2 != Gain::DISABLED)
	{
		writeRegister(Reg::ADC_2, (uint8_t)ch2);
		m_activeChannels += 1;
		channelsMask |= (1 << 1);
	}

	if (ch3 != Gain::DISABLED)
	{
		writeRegister(Reg::ADC_3, (uint8_t)ch3);
		m_activeChannels += 1;
		channelsMask |= (1 << 2);
	}

	if (ch4 != Gain::DISABLED)
	{
		writeRegister(Reg::ADC_4, (uint8_t)ch4);
		m_activeChannels += 1;
		channelsMask |= (1 << 3);
	}

	writeRegister(Reg::ADC_ENA, channelsMask);

	m_frameSize = 1 + m_activeChannels;
}


/** Wakes the chip un, neccessary to call after initial configuration.
 *
 */
void ADS131A0x::wakeup(void)
{
	sendCommand(Command::WAKEUP);
}


/** Writes data into a register.
 *
 * @param reg				Register to write.
 * @param data				Data to write.
 */
void ADS131A0x::writeRegister(Reg reg, uint8_t data) const
{
	sendCommand(((uint32_t)Command::WRITE_REGISTER) | ((uint32_t)reg << 8), data);
}


/** Reads data from a register.
 *
 * @param reg				Register to read.
 * @return					Data read, or 0 in case of wrong read.
 */
uint8_t ADS131A0x::readRegister(Reg reg) const
{
	sendCommand(((uint32_t)Command::READ_REGISTER) | ((uint32_t)reg << 8));

	uint16_t response = sendCommand(Command::NOP);

	if ((response >> 8) == (((uint8_t)reg) | 0b00100000))
		return response;
	else
		return 0;
}


/** Reads data from the ADC using polling.
 *
 * @param data				Pointer to 4 32bit words to store data.
 * @return					STAT_1 register in case of successful read, 0 otherwise.
 */
uint8_t ADS131A0x::readData(int32_t* data)
{
	uint8_t out[MAX_FRAME_SIZE_BYTES] = {};
	uint8_t in[MAX_FRAME_SIZE_BYTES] = {};
	out[0] = (uint32_t)Command::NOP >> 8;
	out[1] = (uint32_t)Command::NOP & 0xFF;

	HAL_GPIO_WritePin(m_cs_port, m_cs_pin, GPIO_PIN_RESET);
	HAL_SPI_TransmitReceive(m_spi, out, in, m_frameSize * m_wordSizeInBytes, ADS131_SPI_TIMEOUT);
	HAL_GPIO_WritePin(m_cs_port, m_cs_pin, GPIO_PIN_SET);

	return processData(in, data);
}


/** Attaches two buffers, which are used for DMA transactions. Also checks for valid DMA configuration.
 *
 * @param buf1				Pointer to  buffer of size MAX_FRAME_SIZE_BYTES.
 * @param buf2				Pointer to  buffer of size MAX_FRAME_SIZE_BYTES.
 * @return					True if configuration is valid.
 */
bool ADS131A0x::attachDmaBuffers(uint8_t* buf1, uint8_t* buf2)
{
	m_dmaOutBuf = buf1;
	m_dmaInBuf = buf2;

	for (size_t i = 0; i < MAX_FRAME_SIZE_BYTES; i++)
		m_dmaOutBuf[i] = 0;


	return ((m_spi->hdmatx != nullptr) && (m_spi->hdmarx != nullptr));
}


/** Starts reading out the data using DMA, version with no operation.
 *
 */
void ADS131A0x::readDataDmaStart()
{
	m_dmaOutBuf[0] = 0;
	m_dmaOutBuf[1] = 0;

	HAL_GPIO_WritePin(m_cs_port, m_cs_pin, GPIO_PIN_RESET);
	HAL_SPI_TransmitReceive_DMA(m_spi, m_dmaOutBuf, m_dmaInBuf, m_frameSize * m_wordSizeInBytes);
}


/** Starts reading out the data using DMA, version for reading a register in the next reply.
 *
 * @param regToRead			Register, whose value will be read out in the next frame.
 */
void ADS131A0x::readDataDmaStart(Reg regToRead)
{
	m_dmaOutBuf[0] = ((uint8_t)((uint16_t)(Command::READ_REGISTER) >> 8)) | (uint8_t)regToRead;
	m_dmaOutBuf[1] = 0;

	HAL_GPIO_WritePin(m_cs_port, m_cs_pin, GPIO_PIN_RESET);
	HAL_SPI_TransmitReceive_DMA(m_spi, m_dmaOutBuf, m_dmaInBuf, m_frameSize * m_wordSizeInBytes);
}


/** Starts reading out the data using DMA, version for writing to a reagister.
 *
 * @param regToWrite		Register, into which data will be written.
 * @param data				Data to write.
 */
void ADS131A0x::readDataDmaStart(Reg regToWrite, uint8_t data)
{
	m_dmaOutBuf[0] = ((uint8_t)((uint16_t)(Command::WRITE_REGISTER) >> 8)) | (uint8_t)regToWrite;
	m_dmaOutBuf[1] = data;

	HAL_GPIO_WritePin(m_cs_port, m_cs_pin, GPIO_PIN_RESET);
	HAL_SPI_TransmitReceive_DMA(m_spi, m_dmaOutBuf, m_dmaInBuf, m_frameSize * m_wordSizeInBytes);
}


/** Processes the data after DMA readout, should be called as DMA TC callback.
 *
 * @param data				4 32 bit words, into which data will be stored.
 * @return					Register readout value, by default STAT_1.
 */
uint8_t ADS131A0x::readDataDmaStop(int32_t* data)
{
	HAL_GPIO_WritePin(m_cs_port, m_cs_pin, GPIO_PIN_SET);

	return processData(m_dmaInBuf, data);
}

/** Sends a command to the ADC.
 *
 * @param command			Command to execute.
 * @param data				Data for the command.
 * @return					Response from the ADC.
 */
uint16_t ADS131A0x::sendCommand(Command command, uint8_t data) const
{
	return sendCommand((uint16_t)command, data);
}


/** Sends a command to the ADC.
 *
 * @param command			Command to execute.
 * @param data				Data for the command.
 * @return					Response from the ADC.
 */
uint16_t ADS131A0x::sendCommand(uint16_t command, uint8_t data) const
{
	uint8_t out[MAX_FRAME_SIZE_BYTES] = {};
	uint8_t in[MAX_FRAME_SIZE_BYTES] = {};
	out[0] = command >> 8;
	out[1] = command | data;

	HAL_GPIO_WritePin(m_cs_port, m_cs_pin, GPIO_PIN_RESET);
	HAL_SPI_TransmitReceive(m_spi, out, in, m_frameSize * m_wordSizeInBytes, ADS131_SPI_TIMEOUT);
	HAL_GPIO_WritePin(m_cs_port, m_cs_pin, GPIO_PIN_SET);

	return (((uint16_t)in[0] << 8) | in[1]);
}


/** Internal function, which processes the array of bytes into 4 data words
 *
 * @param in			Array with source data.
 * @param out			Array of output data.
 * @return				STAT_1 register.
 */
uint8_t ADS131A0x::processData(uint8_t* in, int32_t* out)
{
	if (m_wordSizeInBytes == 3)		//24 bit mode
		for (uint_fast8_t i = 0; i < m_activeChannels; i++)
		{
			int32_t tmp = ((uint32_t)in[i * m_wordSizeInBytes + m_wordSizeInBytes] << 24) | ((uint32_t)in[i * m_wordSizeInBytes + 1 + m_wordSizeInBytes] << 16) |
				((uint32_t)in[i * m_wordSizeInBytes + 2 + m_wordSizeInBytes] << 8);
			out[i] = tmp >> 8;
		}
	else
		return false;			//TODO: other modes

	return in[1];
}
