#ifndef ADS131A0X_H_
#define ADS131A0X_H_

#include "hal_inc.h"

#ifndef ADS131_SPI_TIMEOUT
#define ADS131_SPI_TIMEOUT		25
#endif

class ADS131A0x
{
public:
	enum class Command: uint16_t		//datasheet pg. 52
	{
		NOP 				= 0x0000,			//aka NULL command
		RESET 				= 0x0011,
		STANDBY 			= 0x0022,
		WAKEUP 				= 0x0033,
		LOCK 				= 0x0555,
		UNLOCK 				= 0x0655,
		READ_REGISTER 		= 0x2000,
		READ_REGISTERS 		= READ_REGISTER,
		WRITE_REGISTER 		= 0x4000,
		WRITE_REGISTERS 	= 0x6000
	};

	enum class Reg: uint8_t				//datasheet pg. 59
	{
		ID_MSB 				= 0x00,
		ID_LSB 				= 0x01,
		STAT_1 				= 0x02,
		STAT_P 				= 0x03,
		STAT_N 				= 0x04,
		STAT_S 				= 0x05,
		ERROR_CNT 			= 0x06,
		STAT_M2 			= 0x07,
		A_SYS_CFG 			= 0x0B,
		D_SYS_CFG 			= 0x0C,
		CLK1 				= 0x0D,
		CLK2 				= 0x0E,
		ADC_ENA 			= 0x0F,
		ADC_1 				= 0x11,
		ADC_2 				= 0x12,
		ADC_3 				= 0x13,
		ADC_4 				= 0x14,

		_NONE
	};

	enum class Reference: uint_fast8_t
	{
		INTERNAL_2440_mV,
		INTERNAL_4000_mV,
		EXTERNAL
	};

	enum class ClkSrc: uint_fast8_t
	{
		XTAL,
		SCLK
	};

	enum class ClkDiv: uint8_t
	{
		DIV2 				= 0b001,
		DIV4 				= 0b010,
		DIV6 				= 0b011,
		DIV8 				= 0b100,
		DIV10 				= 0b101,
		DIV12 				= 0b110,
		DIV14 				= 0b111
	};

	enum class Osr: uint8_t			//fData = fMod / osr
	{
		DIV4096,
		DIV2048,
		DIV1024,
		DIV800,
		DIV768,
		DIV512,
		DIV400,
		DIV384,
		DIV256,
		DIV200,
		DIV192,
		DIV128,
		DIV96,
		DIV64,
		DIV48,
		DIV32
	};

	enum class FaultThreshold: uint8_t
	{
		_5_0_PERCENT,
		_7_5_PERCENT,
		_10_0_PERCENT,
		_12_5_PERCENT,
		_15_0_PERCENT,
		_20_0_PERCENT,
		_25_0_PERCENT,
		_30_0_PERCENT,
	};

	enum class Gain: uint8_t
	{
		_1 = 0b000,
		_2 = 0b001,
		_4 = 0b010,
		_8 = 0b011,
		_16 = 0b100,
		DISABLED
	};

	enum class M1		//value of pin M1
	{
		GROUNDED = 3,
		FLOATING = 2,
		IOVDD = 4
	};

	ADS131A0x(SPI_HandleTypeDef* spi, GPIO_TypeDef* cs_port, uint16_t cs_pin, GPIO_TypeDef* rst_port, uint16_t rst_pin, M1 m1);

	bool init();

	void reset(void) const;

	void setClockSource(ClkSrc clkSrc, ClkDiv clkIn, ClkDiv iClk, Osr osr) const;
	void setReferenceVoltage(Reference reference, FaultThreshold faultThreshold) const;
	void enableChannels(Gain ch1 = Gain::_1, Gain ch2 = Gain::_1, Gain ch3 = Gain::_1, Gain ch4 = Gain::_1);
	void wakeup(void);

	void writeRegister(Reg reg, uint8_t data) const;
	uint8_t readRegister(Reg reg) const;

	uint8_t readData(int32_t* data);

	bool attachDmaBuffers(uint8_t* buf1, uint8_t* buf2);
	void readDataDmaStart();
	void readDataDmaStart(Reg regToRead);
	void readDataDmaStart(Reg regToWrite, uint8_t data);
	uint8_t readDataDmaStop(int32_t* data);

	static constexpr size_t MAX_FRAME_SIZE_BYTES = 6 * 4;

private:
	uint16_t sendCommand(Command command, uint8_t data = 0) const;
	uint16_t sendCommand(uint16_t command, uint8_t data = 0) const;

	uint8_t processData(uint8_t* in, int32_t* out);

	SPI_HandleTypeDef* const m_spi;

	GPIO_TypeDef* const m_cs_port;
	const uint16_t m_cs_pin;
	GPIO_TypeDef* const m_rst_port;
	const uint16_t m_rst_pin;

	uint8_t m_frameSize = 1;
	const uint8_t m_wordSizeInBytes;
	uint8_t m_activeChannels;

	//Gain m_channelGains[4];

	uint8_t* m_dmaInBuf;
	uint8_t* m_dmaOutBuf;
};



#endif /* ADS131A0X_H_ */
