/* adc.cpp
 *
 * Created on: Mar 15, 2022
 * Author: Martin Danek, martin@embedblog.eu
 */

#include "adc.h"
#include "main.h"
#include "hal_inc.h"
#include "custom_assert.h"

#include "config.h"
#include "afe.h"
#include "calibration.h"

#include <math.h>

extern SPI_HandleTypeDef hspi2;
extern TIM_HandleTypeDef htim2;
ADS131A0x ads{&hspi2, ADC_CS_GPIO_Port, ADC_CS_Pin, ADC_RST_GPIO_Port, ADC_RST_Pin, ADS131A0x::M1::GROUNDED};

uint8_t dmaBuf1[ADS131A0x::MAX_FRAME_SIZE_BYTES], dmaBuf2[ADS131A0x::MAX_FRAME_SIZE_BYTES];
volatile int32_t prevAdcData[2];
volatile int32_t minAdcData[2];
volatile int32_t maxAdcData[2];
volatile int64_t adcAvgAccumulator[4];
volatile uint64_t adcRmsAccumulator[2];
volatile uint32_t adcSampleCount;
volatile int32_t adcAvg[4];
volatile uint64_t adcRms[2];
volatile uint32_t zeroCrossings[2];
uint32_t zeroCrossingTick;

ADS131A0x::Reg regToRead = ADS131A0x::Reg::_NONE, regToWrite = ADS131A0x::Reg::_NONE;
uint8_t dataToWrite;
volatile uint16_t regReadout;		//lower byte stores register data, upper stores 0xAC for successful readout
volatile bool storeNextReadout;
volatile bool readoutReady;
volatile uint8_t lastStatus;

SampleRate gs_sampleRate;
const char* sampleRates[] = {"1k", "8k", "32k", "64k"};

void adc::init()
{
	calibration::init();

	ASSERT(ads.init());
	ads.setClockSource(ADS131A0x::ClkSrc::XTAL, ADS131A0x::ClkDiv::DIV2, ADS131A0x::ClkDiv::DIV2, ADS131A0x::Osr::DIV4096);		//defaults to 1 kHz output
	ads.setReferenceVoltage(ADS131A0x::Reference::INTERNAL_2440_mV, ADS131A0x::FaultThreshold::_5_0_PERCENT);
	ads.enableChannels(ADS131A0x::Gain::_1, ADS131A0x::Gain::_1, ADS131A0x::Gain::_1, ADS131A0x::Gain::_2);
	ASSERT(ads.attachDmaBuffers(dmaBuf1, dmaBuf2));

	ads.wakeup();

	NVIC_EnableIRQ(ADC_DRDY_EXTI_IRQn);
}


void adc::drdyCb()
{
	if (regToRead != ADS131A0x::Reg::_NONE)
		ads.readDataDmaStart(regToRead);
	else if (regToWrite != ADS131A0x::Reg::_NONE)
	{
		ads.readDataDmaStart(regToWrite, dataToWrite);
		regToWrite = ADS131A0x::Reg::_NONE;
	}
	else
		ads.readDataDmaStart();
}


ATTR_FAST void adc::dmaCpltCb()
{
	HAL_GPIO_WritePin(EXP11_GPIO_Port, EXP11_Pin, GPIO_PIN_SET);

	//get data
	int32_t adcData[4];
	auto readout = ads.readDataDmaStop(adcData);

	//store data
	adcSampleCount++;
	for (uint_fast8_t i = 0; i < 4; i++)
		adcAvgAccumulator[i] += adcData[i];

	for (uint_fast8_t i = 0; i < 2; i++)
	{
		adcRmsAccumulator[i] += (int64_t)adcData[i] * (int64_t)adcData[i];

		//check min/max for voltage/current
		if (minAdcData[i] > adcData[i])
			minAdcData[i] = adcData[i];
		if (maxAdcData[i] < adcData[i])
			maxAdcData[i] = adcData[i];

		//measure frequency
		if ((prevAdcData[i] >= adcAvg[i]) && (adcData[i] < adcAvg[i]))		//detect negative zero-crossing
			zeroCrossings[i]++;
		prevAdcData[i] = adcData[i];
	}

	//process data
	if (adcSampleCount >= 250)
	{
		for (uint_fast8_t i = 0; i < 4; i++)
		{
			adcAvg[i] = adcAvgAccumulator[i] / adcSampleCount;
			adcAvgAccumulator[i] = 0;
		}

		for (uint_fast8_t i = 0; i < 2; i++)
		{
			adcRms[i] = adcRmsAccumulator[i] / adcSampleCount;
			adcRmsAccumulator[i] = 0;
		}

		adcSampleCount = 0;
	}

	//register R/W
	if (storeNextReadout)
	{
		storeNextReadout = false;
		readoutReady = true;
		regReadout = readout;
	}
	else
		lastStatus = readout;

	if (regToRead != ADS131A0x::Reg::_NONE)
	{
		regToRead = ADS131A0x::Reg::_NONE;
		storeNextReadout = true;
	}
	HAL_GPIO_WritePin(EXP11_GPIO_Port, EXP11_Pin, GPIO_PIN_RESET);
}


int32_t adc::getChannelAvgRaw(uint_fast8_t channel)
{
	ASSERT_WITH_RETURN_VALUE(channel < 4, 0);
	return adcAvg[channel];						//'volatile' keyword should ensure access without modification in the middle
}


uint32_t adc::getChannelRmsRaw(uint_fast8_t channel)
{
	ASSERT_WITH_RETURN_VALUE(channel < 2, 0);
	return sqrt(adcRms[channel]);
}


float adc::getChannelDc(uint_fast8_t channel)
{
	float retval = getChannelAvgRaw(channel);
	calibration::applyCalibration(channel, &retval);
	return retval;
}


float adc::getChannelRms(uint_fast8_t channel)
{
	float retval = getChannelRmsRaw(channel);
	calibration::applyCalibration(channel, &retval);
	return retval;
}


float adc::getChannelAcRms(uint_fast8_t channel)
{
	auto dc = getChannelDc(channel);
	auto rms = getChannelRms(channel);

	float root = rms * rms - dc * dc;
	if (root <= 0.0f)
		return 0.0f;
	else
		return (sqrt(root));
}


void adc::getChannelMinMax(uint_fast8_t channel, float* min, float* max)
{
	ASSERT(channel < 2);
	*min = (float)minAdcData[channel];
	*max = (float)maxAdcData[channel];

	calibration::applyCalibration(channel, min);
	calibration::applyCalibration(channel, max);
}


void adc::resetMinMax()
{
	for (uint_fast8_t i = 0; i < 2; i++)
	{
		minAdcData[i] = 0;
		maxAdcData[i] = 0;
	}
}


void adc::doAutoranging()
{
	//voltage autoranging
	if ((maxAdcData[ADC_VOLT_CH] > AUTORANGE_UP) || (minAdcData[ADC_VOLT_CH] < -AUTORANGE_UP))
		afe::increaseVoltageRange(false);
	else if ((maxAdcData[ADC_VOLT_CH] < AUTORANGE_DOWN) && (minAdcData[ADC_VOLT_CH] > -AUTORANGE_UP))
		afe::decreaseVoltageRange(false);

	//current autoranging
	if ((maxAdcData[ADC_CURR_CH] > AUTORANGE_UP) || (minAdcData[ADC_CURR_CH] < -AUTORANGE_UP))
		afe::increaseCurrentRange(false);
	else if ((maxAdcData[ADC_CURR_CH] < AUTORANGE_DOWN) && (minAdcData[ADC_CURR_CH] > -AUTORANGE_UP))
		afe::decreaseCurrentRange(false);
}


float adc::getAnalogFrequency(uint_fast8_t channel)
{
	ASSERT_WITH_RETURN_VALUE(channel < 2, 0);
	float retval = 10000000.0f * zeroCrossings[channel] / (__HAL_TIM_GET_COUNTER(&htim2) - zeroCrossingTick);
	zeroCrossings[channel] = 0;
	zeroCrossingTick = __HAL_TIM_GET_COUNTER(&htim2);
	return retval;
}


uint8_t adc::readRegisterAsync(ADS131A0x::Reg reg)
{
	regToRead = reg;

	while (!readoutReady)
		__asm("nop");
	readoutReady = false;

	return regReadout & 0xFF;
}


uint8_t adc::readLastStatusAsync()
{
	return lastStatus;
}


void adc::writeRegisterAsync(ADS131A0x::Reg reg, uint8_t value)
{
	regToWrite = reg;
	dataToWrite = value;
	while (regToWrite == reg)
		__asm("nop");
}


void adc::increaseSampleRate()
{
	gs_sampleRate = (SampleRate)(((uint32_t)gs_sampleRate) + 1);
	if (gs_sampleRate == SampleRate::__MAX)
		gs_sampleRate = SampleRate::_1_KSPS;

	writeRegisterAsync(ADS131A0x::Reg::CLK2, (uint8_t)gs_sampleRate);
}


SampleRate adc::getSampleRate()
{
	return gs_sampleRate;
}
