/* afe.cpp
 *
 * Created on: Mar 21, 2022
 * Author: Martin Danek, martin@embedblog.eu
 */

#include "afe.h"

#include "hal_inc.h"
#include "custom_assert.h"
#include "main.h"
#include "adc.h"

static VoltageRange gs_vRange;
static CurrentRange gs_cRange;
static AuxResistor gs_aRange;


void afe::setCommonTerminal(CommonTerminal commonTerminal)
{
	switch (commonTerminal)
	{
		case CommonTerminal::FLOATING:
			HAL_GPIO_WritePin(GSEL0_GPIO_Port, GSEL0_Pin, GPIO_PIN_RESET);
			HAL_GPIO_WritePin(GSEL1_GPIO_Port, GSEL1_Pin, GPIO_PIN_RESET);
			break;
		case CommonTerminal::REF_2:
			HAL_GPIO_WritePin(GSEL0_GPIO_Port, GSEL0_Pin, GPIO_PIN_SET);
			HAL_GPIO_WritePin(GSEL1_GPIO_Port, GSEL1_Pin, GPIO_PIN_RESET);
			break;
		case CommonTerminal::GROUNDED:
			HAL_GPIO_WritePin(GSEL0_GPIO_Port, GSEL0_Pin, GPIO_PIN_RESET);
			HAL_GPIO_WritePin(GSEL1_GPIO_Port, GSEL1_Pin, GPIO_PIN_SET);
			break;
		case CommonTerminal::CURRENT_SINK:
			HAL_GPIO_WritePin(GSEL0_GPIO_Port, GSEL0_Pin, GPIO_PIN_SET);
			HAL_GPIO_WritePin(GSEL1_GPIO_Port, GSEL1_Pin, GPIO_PIN_SET);
			break;
		default:
			ASSERT_ALWAYS();
	}
}


void afe::setVoltageRange(VoltageRange voltageRange)
{
	if (gs_vRange == VoltageRange::_60_mV)		//if previous range was 60 mV, set gain to 1
	{
		HAL_GPIO_WritePin(LV_RST_GPIO_Port, LV_RST_Pin, GPIO_PIN_SET);
		adc::writeRegisterAsync(ADS131A0x::Reg::ADC_1, 0);
		HAL_Delay(8);
		HAL_GPIO_WritePin(LV_RST_GPIO_Port, LV_RST_Pin, GPIO_PIN_RESET);
	}

	gs_vRange = voltageRange;

	switch(gs_vRange)
	{
		case VoltageRange::_60_mV:
			HAL_GPIO_WritePin(LV_SET_GPIO_Port, LV_SET_Pin, GPIO_PIN_SET);
			HAL_GPIO_WritePin(VRNG0_GPIO_Port, VRNG0_Pin, GPIO_PIN_SET);
			HAL_GPIO_WritePin(VRNG1_GPIO_Port, VRNG1_Pin, GPIO_PIN_SET);
			adc::writeRegisterAsync(ADS131A0x::Reg::ADC_1, 1);				//set gain to 2
			HAL_Delay(8);
			HAL_GPIO_WritePin(LV_SET_GPIO_Port, LV_SET_Pin, GPIO_PIN_RESET);
			break;
		case VoltageRange::_600_mV:
			HAL_GPIO_WritePin(VRNG0_GPIO_Port, VRNG0_Pin, GPIO_PIN_RESET);
			HAL_GPIO_WritePin(VRNG1_GPIO_Port, VRNG1_Pin, GPIO_PIN_RESET);
			break;
		case VoltageRange::_6_V:
			HAL_GPIO_WritePin(VRNG0_GPIO_Port, VRNG0_Pin, GPIO_PIN_SET);
			HAL_GPIO_WritePin(VRNG1_GPIO_Port, VRNG1_Pin, GPIO_PIN_RESET);
			break;
		case VoltageRange::_60_V:
			HAL_GPIO_WritePin(VRNG0_GPIO_Port, VRNG0_Pin, GPIO_PIN_RESET);
			HAL_GPIO_WritePin(VRNG1_GPIO_Port, VRNG1_Pin, GPIO_PIN_SET);
			break;
		case VoltageRange::_600_V:
			HAL_GPIO_WritePin(VRNG0_GPIO_Port, VRNG0_Pin, GPIO_PIN_SET);
			HAL_GPIO_WritePin(VRNG1_GPIO_Port, VRNG1_Pin, GPIO_PIN_SET);
			break;
		default:
			ASSERT_ALWAYS();
			break;
	}
}


VoltageRange afe::getVoltageRange()
{
	return gs_vRange;
}


void afe::increaseVoltageRange(bool isCircularAllowed)
{
	VoltageRange vr = (VoltageRange)(((uint32_t)gs_vRange) + 1);
	if (vr == VoltageRange::__MAX)
	{
		if (isCircularAllowed)
			vr = VoltageRange::_60_mV;
		else
			return;
	}
	setVoltageRange(vr);
}


void afe::decreaseVoltageRange(bool isCircularAllowed)
{
	VoltageRange vr = (VoltageRange)(((uint32_t)gs_vRange) - 1);
	if (vr == VoltageRange::__MIN)
	{
		if (isCircularAllowed)
			vr = VoltageRange::_600_V;
		else
			return;
	}
	setVoltageRange(vr);
}


void afe::setCurrentRange(CurrentRange currentRange)
{
	if (gs_vRange == VoltageRange::_60_mV)		//if previous range was 6 mA, set gain to 1
		adc::writeRegisterAsync(ADS131A0x::Reg::ADC_2, 0);

	gs_cRange = currentRange;

	switch (gs_cRange)
	{
		case CurrentRange::_6_mA:
			adc::writeRegisterAsync(ADS131A0x::Reg::ADC_2, 1);
			HAL_GPIO_WritePin(IDRV0_GPIO_Port, IDRV0_Pin, GPIO_PIN_RESET);
			HAL_GPIO_WritePin(IDRV1_GPIO_Port, IDRV1_Pin, GPIO_PIN_RESET);
			HAL_GPIO_WritePin(IDRV2_GPIO_Port, IDRV2_Pin, GPIO_PIN_RESET);

			HAL_GPIO_WritePin(IRNG0_GPIO_Port, IRNG0_Pin, GPIO_PIN_SET);
			HAL_GPIO_WritePin(IRNG1_GPIO_Port, IRNG1_Pin, GPIO_PIN_SET);
			break;
		case CurrentRange::_60_mA:
			HAL_GPIO_WritePin(IDRV0_GPIO_Port, IDRV0_Pin, GPIO_PIN_RESET);
			HAL_GPIO_WritePin(IDRV1_GPIO_Port, IDRV1_Pin, GPIO_PIN_RESET);
			HAL_GPIO_WritePin(IDRV2_GPIO_Port, IDRV2_Pin, GPIO_PIN_SET);

			HAL_GPIO_WritePin(IRNG0_GPIO_Port, IRNG0_Pin, GPIO_PIN_SET);
			HAL_GPIO_WritePin(IRNG1_GPIO_Port, IRNG1_Pin, GPIO_PIN_RESET);
			break;
		case CurrentRange::_600_mA:
			HAL_GPIO_WritePin(IDRV0_GPIO_Port, IDRV0_Pin, GPIO_PIN_RESET);
			HAL_GPIO_WritePin(IDRV1_GPIO_Port, IDRV1_Pin, GPIO_PIN_SET);
			HAL_GPIO_WritePin(IDRV2_GPIO_Port, IDRV2_Pin, GPIO_PIN_SET);

			HAL_GPIO_WritePin(IRNG0_GPIO_Port, IRNG0_Pin, GPIO_PIN_RESET);
			HAL_GPIO_WritePin(IRNG1_GPIO_Port, IRNG1_Pin, GPIO_PIN_SET);
			break;
		case CurrentRange::_6_A:
			HAL_GPIO_WritePin(IDRV0_GPIO_Port, IDRV0_Pin, GPIO_PIN_SET);
			HAL_GPIO_WritePin(IDRV1_GPIO_Port, IDRV1_Pin, GPIO_PIN_SET);
			HAL_GPIO_WritePin(IDRV2_GPIO_Port, IDRV2_Pin, GPIO_PIN_SET);

			HAL_GPIO_WritePin(IRNG0_GPIO_Port, IRNG0_Pin, GPIO_PIN_RESET);
			HAL_GPIO_WritePin(IRNG1_GPIO_Port, IRNG1_Pin, GPIO_PIN_RESET);
			break;
		default:
			ASSERT_ALWAYS();
			break;
	}
	HAL_Delay(5);
}


CurrentRange afe::getCurrentRange()
{
	return gs_cRange;
}


void afe::increaseCurrentRange(bool isCircularAllowed)
{
	CurrentRange cr = (CurrentRange)(((uint32_t)gs_cRange) + 1);
	if (cr == CurrentRange::__MAX)
	{
		if (isCircularAllowed)
			cr = CurrentRange::_6_mA;
		else
			return;
	}
	setCurrentRange(cr);
}

void afe::decreaseCurrentRange(bool isCircularAllowed)
{
	CurrentRange cr = (CurrentRange)(((uint32_t)gs_cRange) - 1);
	if (cr == CurrentRange::__MIN)
	{
		if (isCircularAllowed)
			cr = CurrentRange::_6_A;
		else
			return;
	}
	setCurrentRange(cr);
}


void afe::setAuxResistor(AuxResistor auxResistor)
{
	gs_aRange = auxResistor;

	switch (gs_aRange)
	{
		case AuxResistor::NONE:
			HAL_GPIO_WritePin(ARNG0_GPIO_Port, ARNG0_Pin, GPIO_PIN_RESET);
			HAL_GPIO_WritePin(ARNG1_GPIO_Port, ARNG1_Pin, GPIO_PIN_RESET);
			HAL_GPIO_WritePin(ARNG2_GPIO_Port, ARNG2_Pin, GPIO_PIN_RESET);
			HAL_GPIO_WritePin(ARNG3_GPIO_Port, ARNG3_Pin, GPIO_PIN_RESET);
			break;
		case AuxResistor::_510_OHM:
			HAL_GPIO_WritePin(ARNG0_GPIO_Port, ARNG0_Pin, GPIO_PIN_RESET);
			HAL_GPIO_WritePin(ARNG1_GPIO_Port, ARNG1_Pin, GPIO_PIN_RESET);
			HAL_GPIO_WritePin(ARNG2_GPIO_Port, ARNG2_Pin, GPIO_PIN_SET);
			HAL_GPIO_WritePin(ARNG3_GPIO_Port, ARNG3_Pin, GPIO_PIN_SET);
			break;
		case AuxResistor::_1_KILO_OHM:
			HAL_GPIO_WritePin(ARNG0_GPIO_Port, ARNG0_Pin, GPIO_PIN_SET);
			HAL_GPIO_WritePin(ARNG1_GPIO_Port, ARNG1_Pin, GPIO_PIN_RESET);
			HAL_GPIO_WritePin(ARNG2_GPIO_Port, ARNG2_Pin, GPIO_PIN_RESET);
			HAL_GPIO_WritePin(ARNG3_GPIO_Port, ARNG3_Pin, GPIO_PIN_RESET);
			break;
		case AuxResistor::_10_KILO_OHM:
			HAL_GPIO_WritePin(ARNG0_GPIO_Port, ARNG0_Pin, GPIO_PIN_RESET);
			HAL_GPIO_WritePin(ARNG1_GPIO_Port, ARNG1_Pin, GPIO_PIN_SET);
			HAL_GPIO_WritePin(ARNG2_GPIO_Port, ARNG2_Pin, GPIO_PIN_RESET);
			HAL_GPIO_WritePin(ARNG3_GPIO_Port, ARNG3_Pin, GPIO_PIN_RESET);
			break;
		case AuxResistor::_100_KILO_OHM:
			HAL_GPIO_WritePin(ARNG0_GPIO_Port, ARNG0_Pin, GPIO_PIN_SET);
			HAL_GPIO_WritePin(ARNG1_GPIO_Port, ARNG1_Pin, GPIO_PIN_SET);
			HAL_GPIO_WritePin(ARNG2_GPIO_Port, ARNG2_Pin, GPIO_PIN_RESET);
			HAL_GPIO_WritePin(ARNG3_GPIO_Port, ARNG3_Pin, GPIO_PIN_RESET);
			break;
		case AuxResistor::_1_MEGA_OHM:
			HAL_GPIO_WritePin(ARNG0_GPIO_Port, ARNG0_Pin, GPIO_PIN_RESET);
			HAL_GPIO_WritePin(ARNG1_GPIO_Port, ARNG1_Pin, GPIO_PIN_RESET);
			HAL_GPIO_WritePin(ARNG2_GPIO_Port, ARNG2_Pin, GPIO_PIN_SET);
			HAL_GPIO_WritePin(ARNG3_GPIO_Port, ARNG3_Pin, GPIO_PIN_RESET);
			break;
		case AuxResistor::_10_MEGA_OHM:
			HAL_GPIO_WritePin(ARNG0_GPIO_Port, ARNG0_Pin, GPIO_PIN_RESET);
			HAL_GPIO_WritePin(ARNG1_GPIO_Port, ARNG1_Pin, GPIO_PIN_RESET);
			HAL_GPIO_WritePin(ARNG2_GPIO_Port, ARNG2_Pin, GPIO_PIN_RESET);
			HAL_GPIO_WritePin(ARNG3_GPIO_Port, ARNG3_Pin, GPIO_PIN_SET);
			break;
		default:
			ASSERT_ALWAYS();
			break;
	}
}


AuxResistor afe::getAuxResistor()
{
	return gs_aRange;
}


void afe::decreaseAuxResistor()
{
	AuxResistor ar = (AuxResistor)(((uint32_t)gs_aRange) - 1);
	if (ar == AuxResistor::__MIN)
		ar = AuxResistor::_10_MEGA_OHM;
	setAuxResistor(ar);
}
