/* calibration.cpp
 *
 * Created on: April 21, 2022
 * Author: Martin Danek, martin@embedblog.eu
 */

#include "calibration.h"

#include "afe.h"
#include "eeprom24.h"
#include "custom_assert.h"

#include <string.h>

struct CalibrationData
{
	float gain;
	float offset;
};

struct CalibrationStruct
{
	uint32_t magicWord;
	uint32_t timestamp;
	float temperature;

	CalibrationData voltageCalib[VOLTAGE_RANGES];
	CalibrationData currentCalib[CURRENT_RANGES];
	CalibrationData auxCalib;
	CalibrationData tcCalib;
};

CalibrationStruct cs =		//default calibration data, used if nothing found in EEPROM
{
	.magicWord = 0x00,
	.timestamp = 0,
	.temperature = 0.0f,
	.voltageCalib =
	{
		{0.0000086838195529f, -0.0047320012806287f},	//60 mV
		{0.0000838942118250f, -36.1020601384461000f},	//600 mV
		{0.0000010073323920f, -0.0162210735078805f},	//6 V
		{0.0000088283477141f, 0.1097748912964890f},		//60 V
		{1.0f, 0.0f},									//600 V
	},
	.currentCalib =
	{
		{0.0000037296389228f, 0.0204033398523680f},		//6 mA
		{0.0000074738873117f, 0.0606827271272883f},		//60 mA
		{0.0000725614520182f, 0.7317190021207120f},		//600 mA
		{0.0000006413862672f, 0.0334801604241326f},		//6 A
	},
	.auxCalib = {0.0000002916573883f, -0.001885128f},
	.tcCalib = {291.109E-9 / (2 * 40.2157f), 0.0f},
};

constexpr size_t CALIB_SIZE = sizeof(cs);
constexpr uint32_t EEPROM_MAGIC_WORD = 0xBE01EF02;

extern I2C_HandleTypeDef hi2c1;
Eeprom24_512 eeprom{&hi2c1};

void calibration::init()
{
	ASSERT(eeprom.init());
	uint8_t tmp[128];
	eeprom.readPage(0, tmp, CALIB_SIZE);

	memcpy(&cs, tmp, 4);					//copy only the magic word
	if (cs.magicWord == EEPROM_MAGIC_WORD)
	{
		memcpy(&cs, tmp, CALIB_SIZE);		//copy entire calibration struct
	}
}

void calibration::applyCalibration(uint_fast8_t channel, float* value)
{
	size_t index;
	switch (channel) 		//determine which calibration constant to use
	{
		case 0:
			index = (size_t)afe::getVoltageRange() - 1;
			*value *= cs.voltageCalib[index].gain;
			*value += cs.voltageCalib[index].offset;
			break;
		case 1:
			index = (size_t)afe::getCurrentRange() - 1;
			*value *= cs.currentCalib[index].gain;
			*value += cs.currentCalib[index].offset;
			break;
		case 2:
			*value *= cs.auxCalib.gain;
			*value += cs.auxCalib.offset;
			break;
		case 3:
			*value *= cs.tcCalib.gain;
			*value += cs.tcCalib.offset;
			break;
		default:
			break;
	}
}
