/* screen_curr.cpp
 *
 * Created on: May 18, 2022
 * Author: Martin Danek, martin@embedblog.eu
 */

#include "screen_include.h"
#include "SansMono_60p.h"
#include "SansMono_24p.h"
#include "SansMono_24p_Bold.h"
#include "SansMono_16p.h"
#include "SansMono_16p_Bold.h"

#include "adc.h"
#include "afe.h"
#include "config.h"
#include "display.h"

#include "custom_log.h"

const char* iRangeNames[] = {"6 mA", "60 mA", "600 mA", "6 A"};
const char* iPrintFormats[] = {"%5.3f", "%5.2f", "%5.1f", "%5.3f"};
const char* iPrintMinMax[] = {"%5.2fmA%s", "%5.1fmA%s", "%5.0fmA%s", "%5.2fA%s"};
const char* iUnits[] = {"mA", "mA", "mA", "A"};

static uint_fast8_t currentView;
static constexpr uint8_t MAX_VIEW = 4;

static bool isAutoRangeEnabled = false;


static void onUpdate()
{
	lcd.clear();
	drawSystemBar();
	size_t rangeUsed = (size_t)afe::getCurrentRange() - 1;
	float min, max;
	adc::getChannelMinMax(ADC_CURR_CH, &min, &max);

	switch (currentView)
	{
		case 0:
		{
			auto dc = adc::getChannelDc(ADC_CURR_CH);
			if (checkRelMode(&dc))
			{
				lcd.setFont(&SansMono_24p);
				lcd.print(160, 83, "REL");
			}

			lcd.setFont(&SansMono_60p);
			lcd.printf(120, 61, Align::CENTER, iPrintFormats[rangeUsed], dc);

			lcd.setFont(&SansMono_24p_Bold);
			lcd.print(0, 83, Align::LEFT, "DC");
			lcd.printf(240, 83, Align::RIGHT, "%s", iUnits[rangeUsed]);

			lcd.setFont(&SansMono_16p);
			lcd.drawHorLine(0, 84, 240, COLOR_BLACK);
			lcd.printf(0, 100, iPrintMinMax[rangeUsed], min, "min");
			lcd.printf(120, 100, iPrintMinMax[rangeUsed], max, "max");
			LOGI("DC AMP: %.3f %s\n", dc, iUnits[rangeUsed]);
			break;
		}
		case 1:
		{
			auto rms = adc::getChannelRms(ADC_CURR_CH);
			if (checkRelMode(&rms))
			{
				lcd.setFont(&SansMono_24p);
				lcd.print(160, 83, "REL");
			}

			lcd.setFont(&SansMono_60p);
			lcd.printf(120, 61, Align::CENTER, iPrintFormats[rangeUsed], rms);

			lcd.setFont(&SansMono_24p_Bold);
			lcd.print(0, 83, Align::LEFT, "RMS");
			lcd.printf(240, 83, Align::RIGHT, "%s", iUnits[rangeUsed]);

			lcd.setFont(&SansMono_16p);
			lcd.drawHorLine(0, 84, 240, COLOR_BLACK);
			lcd.printf(0, 100, iPrintMinMax[rangeUsed], min, "min");
			lcd.printf(120, 100, iPrintMinMax[rangeUsed], max, "max");
			LOGI("RMS AMP: %.3f %s\n", rms, iUnits[rangeUsed]);
			break;
		}
		case 2:
		{
			auto acrms = adc::getChannelAcRms(ADC_CURR_CH);
			if (checkRelMode(&acrms))
			{
				lcd.setFont(&SansMono_24p);
				lcd.print(160, 83, "REL");
			}

			lcd.setFont(&SansMono_60p);
			lcd.printf(120, 61, Align::CENTER, iPrintFormats[rangeUsed], acrms);

			lcd.setFont(&SansMono_24p_Bold);
			lcd.print(0, 83, Align::LEFT, "AC RMS");
			lcd.printf(240, 83, Align::RIGHT, "%s", iUnits[rangeUsed]);

			lcd.setFont(&SansMono_16p);
			lcd.drawHorLine(0, 84, 240, COLOR_BLACK);
			lcd.printf(0, 100, "F: %u Hz", adc::getAnalogFrequency(ADC_VOLT_CH));
			LOGI("AC RMS AMP: %.3f %s\n", acrms, iUnits[rangeUsed]);
			break;
		}
		case 3:
		{
			lcd.setFont(&SansMono_24p_Bold);
			auto dc = adc::getChannelDc(ADC_CURR_CH);
			lcd.printf(120, 36, Align::CENTER, "DC: %.3f %s", dc, iUnits[rangeUsed]);

			auto rms = adc::getChannelRms(ADC_CURR_CH);
			lcd.printf(120, 61, Align::CENTER, "RMS: %.3f %s", rms, iUnits[rangeUsed]);

			auto acrms = adc::getChannelAcRms(ADC_CURR_CH);
			lcd.printf(120, 84, Align::CENTER, "AC: %.3f %s", acrms, iUnits[rangeUsed]);

			lcd.setFont(&SansMono_16p);
			auto raw = adc::getChannelAvgRaw(ADC_CURR_CH);
			lcd.printf(120, 102, Align::CENTER, "RAW: %li", raw);
			break;
		}
	}

	if (isAutoRangeEnabled)
	{
		adc::doAutoranging();
		lcd.setFont(&SansMono_24p);
		lcd.print(80, 83, "AUTO");
	}

	lcd.drawRectFilled(0, 104, 240, 120 - 104, COLOR_BLACK);
	lcd.setPrimaryColor(COLOR_WHITE);
	lcd.setBgColor(COLOR_BLACK);
	lcd.setFont(&SansMono_16p);
	lcd.print(20, 118, Align::CENTER, "2/6");
	lcd.printf(90, 118, Align::CENTER, "%u/4", currentView + 1);
	lcd.print(150, 118, Align::CENTER, iRangeNames[rangeUsed]);
	lcd.print(240, 118, Align::RIGHT, sampleRates[(size_t)adc::getSampleRate()]);
	lcd.setPrimaryColor(COLOR_BLACK);
	lcd.setBgColor(COLOR_WHITE);

	lcd.displayDma();

	adc::resetMinMax();
}


static void onEntry()
{
	lcd.clear();
	lcd.setFont(&SansMono_24p_Bold);
	lcd.printf(120, 90, Align::CENTER, "CURRENT MODE");
	lcd.displayDma();

	HAL_Delay(SCREEN_CHANGE_TIME);
}


static void onAction(const ScreenAction action)
{
	switch (action)
	{
		case ScreenAction::BTN1_PRESS:
			changeScreen(&screen_power);
			break;
		case ScreenAction::BTN2_PRESS:
			currentView++;
			if (currentView >= MAX_VIEW)
				currentView = 0;
			break;
		case ScreenAction::BTN3_PRESS:
			if (isAutoRangeEnabled)
				isAutoRangeEnabled = false;
			else if (afe::getCurrentRange() == CurrentRange::_6_A)
				isAutoRangeEnabled = true;
			else
				afe::increaseCurrentRange(true);
			break;
		case ScreenAction::BTN4_PRESS:
			adc::increaseSampleRate();
			break;
		default:
			break;
	}
}


const Screen screen_curr =
{
	.onUpdate = onUpdate,
	.onEntry = onEntry,
	.onAction = onAction
};
