/* screen_volt.cpp
 *
 * Created on: 15. 3. 2022
 * Author: Martin Danek, martin@embedblog.eu
 */

#include "screen_include.h"
#include "SansMono_60p.h"
#include "SansMono_24p.h"
#include "SansMono_24p_Bold.h"
#include "SansMono_16p.h"
#include "SansMono_16p_Bold.h"

#include "adc.h"
#include "afe.h"
#include "config.h"
#include "display.h"

#include "custom_log.h"

const char* vRangeNames[] = {"60 mV", "600 mV", "6 V", "60 V", "600 V"};
const char* vPrintFormats[] = {"%5.2f", "%5.1f", "%5.3f", "%5.2f", "%5.1f"};
const char* vPrintMinMax[] = {"%5.1fmV%s", "%5.0fmV%s", "%5.2fV%s", "%5.1fV%s", "%5.0fV%s"};
const char* vUnits[] = {"mV", "mV", "V", "V", "V"};

static uint_fast8_t currentView;
static constexpr uint8_t MAX_VIEW = 4;

static bool isAutoRangeEnabled = false;


static void onUpdate()
{
	lcd.clear();
	drawSystemBar();
	size_t rangeUsed = (size_t)afe::getVoltageRange() - 1;
	float min, max;
	adc::getChannelMinMax(ADC_VOLT_CH, &min, &max);

	switch (currentView)
	{
		case 0:
		{
			auto dc = adc::getChannelDc(ADC_VOLT_CH);
			if (checkRelMode(&dc))
			{
				lcd.setFont(&SansMono_24p);
				lcd.print(160, 83, "REL");
			}

			lcd.setFont(&SansMono_60p);
			lcd.printf(120, 61, Align::CENTER, vPrintFormats[rangeUsed], dc);

			lcd.setFont(&SansMono_24p_Bold);
			lcd.print(0, 83, Align::LEFT, "DC");
			lcd.printf(240, 83, Align::RIGHT, "%s", vUnits[rangeUsed]);

			lcd.setFont(&SansMono_16p);
			lcd.drawHorLine(0, 84, 240, COLOR_BLACK);
			lcd.printf(0, 100, vPrintMinMax[rangeUsed], min, "min");
			lcd.printf(120, 100, vPrintMinMax[rangeUsed], max, "max");
			LOGI("DC VOLT: %.3f %s\n", dc, vUnits[rangeUsed]);
			break;
		}
		case 1:
		{
			auto rms = adc::getChannelRms(ADC_VOLT_CH);
			if (checkRelMode(&rms))
			{
				lcd.setFont(&SansMono_24p);
				lcd.print(160, 83, "REL");
			}

			lcd.setFont(&SansMono_60p);
			lcd.printf(120, 61, Align::CENTER, vPrintFormats[rangeUsed], rms);

			lcd.setFont(&SansMono_24p_Bold);
			lcd.print(0, 83, Align::LEFT, "RMS");
			lcd.printf(240, 83, Align::RIGHT, "%s", vUnits[rangeUsed]);

			lcd.setFont(&SansMono_16p);
			lcd.drawHorLine(0, 84, 240, COLOR_BLACK);
			lcd.printf(0, 100, vPrintMinMax[rangeUsed], min, "min");
			lcd.printf(120, 100, vPrintMinMax[rangeUsed], max, "max");
			LOGI("RMS VOLT: %.3f %s\n", rms, vUnits[rangeUsed]);
			break;
		}
		case 2:
		{
			auto acrms = adc::getChannelAcRms(ADC_VOLT_CH);
			if (checkRelMode(&acrms))
			{
				lcd.setFont(&SansMono_24p);
				lcd.print(160, 83, "REL");
			}

			lcd.setFont(&SansMono_60p);
			lcd.printf(120, 61, Align::CENTER, vPrintFormats[rangeUsed], acrms);

			lcd.setFont(&SansMono_24p_Bold);
			lcd.print(0, 83, Align::LEFT, "AC RMS");
			lcd.printf(240, 83, Align::RIGHT, "%s", vUnits[rangeUsed]);

			lcd.setFont(&SansMono_16p);
			lcd.drawHorLine(0, 84, 240, COLOR_BLACK);
			lcd.printf(0, 100, "F: %u Hz", adc::getAnalogFrequency(ADC_VOLT_CH));
			LOGI("AC RMS VOLT: %.3f %s\n", acrms, vUnits[rangeUsed]);
			break;
		}
		case 3:
		{
			lcd.setFont(&SansMono_24p_Bold);
			auto dc = adc::getChannelDc(ADC_VOLT_CH);
			lcd.printf(120, 36, Align::CENTER, "DC: %.3f %s", dc, vUnits[rangeUsed]);

			auto rms = adc::getChannelRms(ADC_VOLT_CH);
			lcd.printf(120, 61, Align::CENTER, "RMS: %.3f %s", rms, vUnits[rangeUsed]);

			auto acrms = adc::getChannelAcRms(ADC_VOLT_CH);
			lcd.printf(120, 84, Align::CENTER, "AC: %.3f %s", acrms, vUnits[rangeUsed]);

			lcd.setFont(&SansMono_16p);
			auto raw = adc::getChannelAvgRaw(ADC_VOLT_CH);
			lcd.printf(120, 102, Align::CENTER, "RAW: %li", raw);
			break;
		}
	}

	if (isAutoRangeEnabled)
	{
		adc::doAutoranging();
		lcd.setFont(&SansMono_24p);
		lcd.print(80, 83, "AUTO");
	}

	lcd.drawRectFilled(0, 104, 240, 120 - 104, COLOR_BLACK);
	lcd.setPrimaryColor(COLOR_WHITE);
	lcd.setBgColor(COLOR_BLACK);
	lcd.setFont(&SansMono_16p);
	lcd.print(20, 118, Align::CENTER, "1/6");
	lcd.printf(90, 118, Align::CENTER, "%u/4", currentView + 1);
	lcd.print(150, 118, Align::CENTER, vRangeNames[rangeUsed]);
	lcd.print(240, 118, Align::RIGHT, sampleRates[(size_t)adc::getSampleRate()]);
	lcd.setPrimaryColor(COLOR_BLACK);
	lcd.setBgColor(COLOR_WHITE);

	lcd.displayDma();

	adc::resetMinMax();
}


static void onEntry()
{
	lcd.clear();
	lcd.setFont(&SansMono_24p_Bold);
	lcd.printf(120, 90, Align::CENTER, "VOLTAGE MODE");
	lcd.displayDma();

	HAL_Delay(SCREEN_CHANGE_TIME);
}


static void onAction(const ScreenAction action)
{
	switch (action)
	{
		case ScreenAction::BTN1_PRESS:
			changeScreen(&screen_curr);
			break;
		case ScreenAction::BTN2_PRESS:
			currentView++;
			if (currentView >= MAX_VIEW)
				currentView = 0;
			break;
		case ScreenAction::BTN3_PRESS:
			if (isAutoRangeEnabled)
				isAutoRangeEnabled = false;
			else if (afe::getVoltageRange() == VoltageRange::_600_V)
				isAutoRangeEnabled = true;
			else
				afe::increaseVoltageRange(true);
			break;
		case ScreenAction::BTN4_PRESS:
			adc::increaseSampleRate();
			break;
		default:
			break;
	}
}

const Screen screen_volt =
{
	.onUpdate = onUpdate,
	.onEntry = onEntry,
	.onAction = onAction
};


