/* main.cpp
 *
 * Created on: Mar 10, 2022
 * Author: Martin Danek, martin@embedblog.eu
 */

#include "main.h"

#include "config.h"
#include "PCA9554.h"
#include "custom_assert.h"
#include "custom_log.h"
#include "display.h"
#include "adc.h"
#include "sys.h"
#include "afe.h"
#include "version.h"
#include "calibration.h"

extern I2C_HandleTypeDef hi2c1;
extern UART_HandleTypeDef huart1;

PCA9554 ioexp{&hi2c1, PCA9554::I2C_ADDRESS_A_VERSION};

volatile ScreenAction pendingAction = ScreenAction::NO_ACTION;

int main()
{
	HAL_Init();
	SystemClock_Config();

	MX_GPIO_Init();
	NVIC_DisableIRQ(ADC_DRDY_EXTI_IRQn);

	HAL_GPIO_WritePin(PWR_EN_GPIO_Port, PWR_EN_Pin, GPIO_PIN_RESET);

	MX_DMA_Init();
	MX_USART1_UART_Init();
	MX_I2C1_Init();
	MX_SPI2_Init();
	MX_RTC_Init();
	MX_TIM2_Init();
	MX_TIM4_Init();
	MX_ADC1_Init();
	MX_DAC1_Init();
	MX_DAC3_Init();
	MX_COMP3_Init();

	ASSERT(ioexp.init());
	sys::init();

	sys::setLed(0, 25);

	HAL_Delay(500);
	for (auto i = 0; i < 6; i++)
	{
		HAL_GPIO_TogglePin(BUZZ_GPIO_Port, BUZZ_Pin);
		HAL_Delay(50);
	}

	LOGI("\n\nMultimeter R3.1 started\n");
	LOGI("FW tag: %s\n", _V_BUILD_TAG);
	LOGI("FW commit: %s\n", _V_COMMIT);
	LOGI("Martin Danek, 2022\n");

	display::init();
	calibration::init();
	afe::setCommonTerminal(CommonTerminal::REF_2);
	afe::setVoltageRange(VoltageRange::_6_V);
	afe::setCurrentRange(CurrentRange::_600_mA);
	adc::init();

	auto lastDisplay = HAL_GetTick();
	while (true)
	{
		if (HAL_GetTick() - lastDisplay > SCREEN_REFRESH_MS)
		{
			lastDisplay = HAL_GetTick();
			display::update();
		}

		if (pendingAction != ScreenAction::NO_ACTION)
		{
			display::action(pendingAction);
			pendingAction = ScreenAction::NO_ACTION;
		}
	}
}


uint8_t prevPort = 0xFF;
void ioExpIsr()
{
	uint8_t newPort = ioexp.readPort();
	uint8_t fallingEdge = ~newPort & prevPort;				//here '1' means 1->0 transition from last read

	if (IOEXP_BTN_1_MASK & fallingEdge)
		pendingAction = ScreenAction::BTN1_PRESS;
	else if (IOEXP_BTN_2_MASK & fallingEdge)
		pendingAction = ScreenAction::BTN2_PRESS;
	else if (IOEXP_BTN_3_MASK & fallingEdge)
		pendingAction = ScreenAction::BTN3_PRESS;
	else if (IOEXP_BTN_4_MASK & fallingEdge)
		pendingAction = ScreenAction::BTN4_PRESS;
	else if (IOEXP_BTN_5_MASK & fallingEdge)
		pendingAction = ScreenAction::BTN5_PRESS;
	else if (IOEXP_BTN_6_MASK & fallingEdge)
		pendingAction = ScreenAction::BTN6_PRESS;
	else if (IOEXP_BTN_7_MASK & fallingEdge)
		pendingAction = ScreenAction::BTN7_PRESS;
	else if (IOEXP_BTN_8_MASK & fallingEdge)
		pendingAction = ScreenAction::BTN8_PRESS;

	prevPort = newPort;
}


extern "C"
{
	void HAL_GPIO_EXTI_Callback(uint16_t pin)
	{
		switch (pin)
		{
			case EXP_IRQ_Pin:
				ioExpIsr();
				break;
			case ADC_DRDY_Pin:
				adc::drdyCb();
				break;
			default: ASSERT_ALWAYS();
		}
	}

	void HAL_SPI_TxRxCpltCallback(SPI_HandleTypeDef* hspi)
	{
		//if (hspi == &hspi2)
		adc::dmaCpltCb();
	}
}


void custom_assert(const char* file, uint32_t line)
{
	__asm("bkpt 0");		//breakpoint instruction
	while (true);
}

void _log_send(const char* msg, size_t len)
{
	HAL_UART_Transmit_DMA(&huart1, (const uint8_t*)msg, len);
}

bool _log_isReady()
{
	return (huart1.gState == HAL_UART_STATE_READY);
}


