/* sys.cpp
 *
 * Created on: Mar 19, 2022
 * Author: Martin Danek, martin@embedblog.eu
 */

#include "sys.h"

#include "TMP75.h"
#include "custom_assert.h"
#include "config.h"
#include "main.h"

extern I2C_HandleTypeDef hi2c1;
extern ADC_HandleTypeDef hadc1;
extern RTC_HandleTypeDef hrtc;
extern TIM_HandleTypeDef htim4;

TMP75 tempSensor{&hi2c1};


void sys::init()
{
	ASSERT(tempSensor.init(TMP75::Resolution::BITS_12));

	RTC_TimeTypeDef tm =
	{
		.Hours = 10,
		.Minutes = 30,
	};
	HAL_RTC_SetTime(&hrtc, &tm, RTC_FORMAT_BIN);
}


float sys::getTemperature()
{
	return tempSensor.readTemperature();
}


//we do an actual measurement only every 10th request
static uint8_t requestCounter = 0xFF;
static uint_fast8_t prevValue;
uint_fast8_t sys::getBatteryPercentage(void)
{
	requestCounter++;
	if (!(requestCounter % 10))		//every 10th request
	{
		HAL_ADC_Start(&hadc1);
		auto start = HAL_GetTick();
		while (!__HAL_ADC_GET_FLAG(&hadc1, ADC_FLAG_EOC))
			if (HAL_GetTick() - start > 25)
				return 0;

		auto data = HAL_ADC_GetValue(&hadc1);
		uint16_t millivolts = ADC_INTERNAL_LSB_SIZE * data * 2 * 1000;

		uint_fast8_t percent;
		if (millivolts <= 3350)
			percent = 0;
		else
			percent = (millivolts - 3350) / 5;

		if (percent > 100)
			percent = 100;

		prevValue = percent;
	}

	return prevValue;
}


void sys::readRtc(uint_fast8_t* hour, uint_fast8_t* minute, uint_fast8_t* second)
{
	RTC_TimeTypeDef tm;
	RTC_DateTypeDef dm;
	HAL_RTC_GetTime(&hrtc, &tm, RTC_FORMAT_BIN);
	HAL_RTC_GetDate(&hrtc, &dm, RTC_FORMAT_BIN);
	*hour = tm.Hours;
	*minute = tm.Minutes;
	*second = tm.Seconds;
}


/** Sets the value for RGB led
 *
 * @param b		Blue value, 0-100.
 * @param g		Green value, 0-100.
 *
 * @note		The PWM frequency is 100 Hz.
 */
void sys::setLed(uint_fast8_t b, uint_fast8_t g)
{
	__HAL_TIM_SET_COMPARE(&htim4, TIM_CHANNEL_1, g);
	__HAL_TIM_SET_COMPARE(&htim4, TIM_CHANNEL_2, b);
}


void sys::powerDown()
{
	setLed(0, 0);
	HAL_GPIO_WritePin(PWR_EN_GPIO_Port, PWR_EN_Pin, GPIO_PIN_SET);
	HAL_PWREx_EnterSTOP1Mode(PWR_STOPENTRY_WFE);

	//after wakeup
	setLed(0, 25);
	HAL_GPIO_WritePin(PWR_EN_GPIO_Port, PWR_EN_Pin, GPIO_PIN_RESET);
	HAL_Delay(5);
}

