#include "UC1608.h"

#define UC1608_SW_RESET			0xE2

#define UC1608_SET_PAGEADD 		0xB0 	//Page address Set PA[3:0]
#define UC1608_SET_COLADD_LSB 	0x00 	//Column Address Set CA [3:0]
#define UC1608_SET_COLADD_MSB 	0x10 	//Column Address Set CA [7:4]
#define UC1608_RAM_ACCESS		0x88	//RAM Access Controll (Wrap Around)

#define UC1608_SET_TC_AND_MR	0x20	//Set Temperature Compensation and Multiplex rate

#define UC1608_GN_PM 			0x81 	//Set V BIAS Potentiometer to fine tune V-D and V-LCD  (double-byte command)
#define UC1608_LCD_CONTROL		0xC0
#define UC1608_SET_BIAS_RATIO	0xE8
#define UC1608_SET_POWER_CTRL	0x28

#define UC1608_ENABLE			0xAE
#define UC1608_INVERT_DISPLAY 	0xA6
#define UC1608_ALL_PIXEL_ON 	0xA4


UC1608::UC1608(	SPI_HandleTypeDef* spi, GPIO_TypeDef* cs_port, uint16_t cs_pin, GPIO_TypeDef* dc_port, uint16_t dc_pin,
				GPIO_TypeDef* rst_port, uint16_t rst_pin, uint16_t width, uint16_t height, uint8_t* displayBuffer, uint32_t displaySize):
					GFX_mono(width, height, displayBuffer, displaySize), m_spi(spi), m_cs_port(cs_port), m_cs_pin(cs_pin), m_dc_port(dc_port),
					m_dc_pin(dc_pin), m_rst_port(rst_port), m_rst_pin(rst_pin)
{};


/** Initializes the display.
 *
 * @param potentiometer	Sets the contrast voltage.
 *
 * @note				Max SPI speed 7 MHz.
 */
void UC1608::init(uint8_t potentiometer)
{
	while (HAL_GetTick() < 15);

	reset();

	sendCommand(UC1608_SET_TC_AND_MR | 0x7);		//set temperature compensation and multiplex rate
	sendCommand(UC1608_SET_POWER_CTRL | 0b111);		//set power control (ie panel loading and Vlcd source)
	sendCommand(UC1608_SET_BIAS_RATIO | 0b10);
	sendCommand(UC1608_RAM_ACCESS | 0b1);			//enable automatic wrap-around

	setRotation(0);
	setVlcd(1, potentiometer);

	setPrimaryColor(COLOR_BLACK);
	setBgColor(COLOR_WHITE);

	enable(true);
}


/** Resets the LCD, either using a GPIO pin or SW reset.
 *
 */
void UC1608::reset(void)
{
	if (m_rst_port != nullptr)
	{
		HAL_Delay(1);
		HAL_GPIO_WritePin(m_rst_port, m_rst_pin, GPIO_PIN_RESET);
		HAL_Delay(5);
		HAL_GPIO_WritePin(m_rst_port, m_rst_pin, GPIO_PIN_SET);
	}
	else
		sendCommand(UC1608_SW_RESET);
}


/** Sets the rotation of the display.
 *
 * @param r
 */
void UC1608::setRotation(uint8_t r)		//TODO: test rotations
{
	m_rotation = r % 4;
	uint8_t rotateReg;
	uint16_t width_temp = m_width;
	uint16_t height_temp = m_height;

	switch (r)
	{
		default:
		case 0:
			rotateReg = 0x04;
			m_width = width_temp;
			m_height = height_temp;
			break;
		case 1:
			rotateReg = 0x02;
			m_height = width_temp;
			m_width = height_temp;
			break;
		case 2:
			rotateReg = 0x06;
			m_width = width_temp;
			m_height = height_temp;
			break;
		case 3:
			rotateReg = 0x00;
			m_height = width_temp;
			m_width = height_temp;
			break;
	}

	sendCommand(UC1608_LCD_CONTROL | rotateReg);
}


/** Sets the contrast of the display.
 *
 * @param gain
 * @param pm
 */
void UC1608::setVlcd(uint8_t gain, uint8_t pm)
{
	if (gain > 3)
		gain = 3;
	if (pm > 63)
		pm = 63;

	uint8_t cmds[] = {UC1608_GN_PM, (uint8_t)((gain << 6) | pm)};
	sendCommand(cmds, 2);
}


/** Enables the display.
 *
 * @param enabled
 */
void UC1608::enable(bool enabled)
{
	sendCommand(UC1608_ENABLE | enabled);
}


/** Turns on color inversion.
 *
 * @param inverted
 */
void UC1608::invert(bool inverted)
{
	sendCommand(UC1608_INVERT_DISPLAY | inverted);
}


/** Turns on all pixels on the display.
 *
 * @param allOn
 */
void UC1608::allPixelsOn(bool allOn)
{
	sendCommand(UC1608_ALL_PIXEL_ON| allOn);
}


/** Pushes data to the display.
 *
 */
void UC1608::display(void)
{
	sendCommand(UC1608_SET_COLADD_LSB | 0);
	sendCommand(UC1608_SET_COLADD_MSB | 0);
	sendCommand(UC1608_SET_PAGEADD | 0);

	if (m_cs_port != nullptr)
		HAL_GPIO_WritePin(m_cs_port, m_cs_pin, GPIO_PIN_SET);

	HAL_GPIO_WritePin(m_dc_port, m_dc_pin, GPIO_PIN_SET);
	HAL_SPI_Transmit(m_spi, (uint8_t*)m_dispBuffer, m_bufPosMax, UC1608_SPI_TIMEOUT);

	if (m_cs_port != nullptr)
		HAL_GPIO_WritePin(m_cs_port, m_cs_pin, GPIO_PIN_RESET);
}


/** Pushes data to the display.
 *
 */
void UC1608::displayDma(void)
{
	auto start = HAL_GetTick();
	while (isDmaInProgress())
	{
		HAL_Delay(1);

		if (HAL_GetTick() - start > UC1608_SPI_TIMEOUT)
			return;
	}

	sendCommand(UC1608_SET_COLADD_LSB | 0);
	sendCommand(UC1608_SET_COLADD_MSB | 0);
	sendCommand(UC1608_SET_PAGEADD | 0);

	if (m_cs_port != nullptr)
		HAL_GPIO_WritePin(m_cs_port, m_cs_pin, GPIO_PIN_SET);

	HAL_GPIO_WritePin(m_dc_port, m_dc_pin, GPIO_PIN_SET);
	HAL_SPI_Transmit_DMA(m_spi, (uint8_t*)m_dispBuffer, m_bufPosMax);
}


/** Checks if DMA transition is still in progress.
 *
 * @return		True if DMA is still transmitting.
 */
bool UC1608::isDmaInProgress(void)
{
	return !(m_spi->hdmatx->State == HAL_DMA_STATE_READY);
}


void UC1608::sendCommand(uint8_t command)
{
	if (m_cs_port != nullptr)
		HAL_GPIO_WritePin(m_cs_port, m_cs_pin, GPIO_PIN_SET);

	HAL_GPIO_WritePin(m_dc_port, m_dc_pin, GPIO_PIN_RESET);
	HAL_SPI_Transmit(m_spi, &command, 1, UC1608_SPI_TIMEOUT);

	if (m_cs_port != nullptr)
		HAL_GPIO_WritePin(m_cs_port, m_cs_pin, GPIO_PIN_RESET);
}


void UC1608::sendCommand(uint8_t* p_cmds, uint8_t num)
{
	if (m_cs_port != nullptr)
		HAL_GPIO_WritePin(m_cs_port, m_cs_pin, GPIO_PIN_SET);
	HAL_GPIO_WritePin(m_dc_port, m_dc_pin, GPIO_PIN_RESET);
	HAL_SPI_Transmit(m_spi, p_cmds, num, UC1608_SPI_TIMEOUT);

	if (m_cs_port != nullptr)
		HAL_GPIO_WritePin(m_cs_port, m_cs_pin, GPIO_PIN_RESET);
}
