/* PCA9554.cpp
 *
 * Created on: May 13, 2022
 * Author: Martin Danek, martin@embedblog.eu
 */

#include "PCA9554.h"

/** Constructor, with option to set 7-bit address.
 *
 * @param i2c		Pointer to already initialized i2c peripheral.
 * @param address	7-bit I2C address.
 *
 * @note			Max I2C clock speed: 400 kHz.
 * @note			For 'A' version of the chip, use I2C_ADDRESS_A_VERSION.
 */
PCA9554::PCA9554(I2C_HandleTypeDef* i2c, uint8_t address):
	m_i2c(i2c),
	m_i2c_address(address << 1)
{};


/** Constructor for setting the address with Address Pins.
 *
 * @param i2c		Pointer to already initialized i2c peripheral.
 * @param A0		State of A0 pin, low = connected to ground.
 * @param A1		State of A1 pin, low = connected to ground.
 * @param A2		State of A2 pin, low = connected to ground.
 *
 * @note			Max I2C clock speed: 400 kHz.
 */
PCA9554::PCA9554(I2C_HandleTypeDef* i2c, bool A0, bool A1, bool A2):
	m_i2c(i2c),
	m_i2c_address((DEFAULT_I2C_ADDRESS | (A0) | (A1 << 1) | (A2 << 2)) << 1)
{};


/** Initializes device, verifies connection and reads default pin state.
 *
 * @return		True if connection was established
 */
bool PCA9554::init()
{
	//just to verify connection
	uint8_t tmp;
	auto retval = HAL_I2C_Master_Receive(m_i2c, m_i2c_address, &tmp, 1, PCA9554_I2C_TIMEOUT);
	return (retval == HAL_OK);
}


/** Reads the input register..
 *
 * @return		Pin values, where 1 means logic high and vice versa.
 */
uint8_t PCA9554::readPort(void) const
{
	uint8_t tmp = (uint8_t)Reg::INPUT_DATA;
	HAL_I2C_Master_Transmit(m_i2c, m_i2c_address, &tmp, 1, PCA9554_I2C_TIMEOUT);
	HAL_I2C_Master_Receive(m_i2c, m_i2c_address, &tmp, 1, PCA9554_I2C_TIMEOUT);
	return tmp;
}


/** Reads a single pin.
 *
 * @param pinNumber		Pin number to read, range 0 - 7.
 * @return				True means that the pin is at logic high and vice versa.
 */
bool PCA9554::readPin(uint8_t pinNumber) const
{
	uint8_t retval = readPort();
	return (retval & (1 << pinNumber));
}
