#include "TMP75.h"


/** Constructor, with option to set 7-bit address.
 *
 * @param i2c		Pointer to already initialized i2c peripheral.
 * @param address	7-bit I2C address.
 *
 *  @note			Max I2C clock speed: 400 kHz.
 */
TMP75::TMP75(I2C_HandleTypeDef* i2c, uint8_t address):
	m_i2c(i2c),
	m_i2c_address(address << 1)
{};


/** Constructor for setting the address with Address Pins.
 *
 * @param i2c		Pointer to already initialized i2c peripheral.
 * @param A0		State of A0 pin, low = connected to ground.
 * @param A1		State of A1 pin, low = connected to ground.
 * @param A2		State of A2 pin, low = connected to ground.
 *
 * @note			Max I2C clock speed: 400 kHz.
 */
TMP75::TMP75(I2C_HandleTypeDef* i2c, bool A0, bool A1, bool A2):
	m_i2c(i2c),
	m_i2c_address((DEFAULT_I2C_ADDRESS | (A0) | (A1 << 1) | (A2 << 2)) << 1)
{};


/** Initializes the chip, verifies connection and sets resolution.
 *
 * @param resolution	Measurement resolution, see the respective enum.
 * @return				True if chip responded to I2C calls.
 */
bool TMP75::init(Resolution resolution)
{
	uint8_t test[2];		//dummy read to verify connection
	auto retval = HAL_I2C_Master_Receive(m_i2c, m_i2c_address, test, sizeof(test), TMP75_I2C_TIMEOUT);

	setResolution(resolution);

	return (retval == HAL_OK);
}


/** Reads the raw temperature from the internal ADC.
 *
 * @return		Raw temperature in the 12 LSb's, units depend on current resolution.
 */
uint16_t TMP75::readTemperatureRaw(void) const
{
	uint8_t data[2] = {(uint8_t)Reg::TEMPERATURE, 0};
	HAL_I2C_Master_Transmit(m_i2c, m_i2c_address, data, 1, TMP75_I2C_TIMEOUT);
	HAL_I2C_Master_Receive(m_i2c, m_i2c_address, data, sizeof(data), TMP75_I2C_TIMEOUT);

	uint16_t raw = ((uint16_t)data[0] << 4) | (data[1] >> 4);		//form a 12 bit word

	return raw >> (3 - (uint_fast8_t)m_resolution);
}


/** Reads the temperature and returns it in human-readable format.
 *
 * @return		Temperature in degrees celsius.
 */
float TMP75::readTemperature(void)
{
	uint16_t raw = readTemperatureRaw() & 0xFFF;

	if (raw > 0x7FF)
		return raw * lsbToDegrees[(uint8_t)m_resolution] * -1.0f;
	else
		return raw * lsbToDegrees[(uint8_t)m_resolution];
}


/** Sets ADC resolution.
 *
 * @param resolution	See the enum.
 */
void TMP75::setResolution(Resolution resolution)
{
	m_resolution = resolution;
	setConfigurationRegister();
}


/** Shuts down the IC, forcing it to enter a shutdown mode with consumption of less than 0.1 uA.
 *
 * @param isEnabled		True to enable shutdown mode, false to disable.
 *
 * @note				This stops regular conversions, so singleConversion function must be called to make a conversion.
 */
void TMP75::shutdown(bool isEnabled)
{
	m_isShutDown = isEnabled;
	setConfigurationRegister();
}


/** Start a single conversion in shutdown mode. Data must be read after conversion time using readData function.
 *
 * @note				Conversion time (ds pg. 20): 30 ms @ 9 bits, 55 ms @ 10 bits, 110 ms @ 11 bits, 220 ms @ 12 bits.
 */
void TMP75::singleConversion()
{
	setConfigurationRegister(true);
}


/** Internal function, sets configuration register based on private variables.
 *
 * @param startSingleConversion		Sets the OS bit to 1, therefore starting a single conversion.
 */
void TMP75::setConfigurationRegister(bool startSingleConversion) const
{
	uint8_t config[2] = {(uint8_t)Reg::CONFIGURATION, (uint8_t)(((uint8_t)m_resolution << 5) | (startSingleConversion << 7) | (m_isShutDown << 0))};

	HAL_I2C_Master_Transmit(m_i2c, m_i2c_address, config, sizeof(config), TMP75_I2C_TIMEOUT);
}
